# lecture5.py


# For complete documentation see:
# https://developer.leapmotion.com/documentation/v2/python/index.html

# common import
import sys
sys.path.append('..')
from common.core import *
from common.gfxutil import *
from common.leaputil import *

from kivy.uix.widget import Widget
from kivy.graphics import Color, Ellipse, Line
from kivy.graphics.instructions import InstructionGroup

import numpy as np

import Leap

# get basic Leap system working - showing palm data from one of the hands
class MainWidget1(BaseWidget) :
    def __init__(self):
        super(MainWidget1, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()

    def on_update(self) :
        leap_frame = self.leap.frame()
        pt = leap_one_palm(leap_frame)
        self.label.text = leap_info(self.leap)
        self.label.text += 'x:%d\ny:%d\nz:%d\n' % (pt[0], pt[1], pt[2])



# convert the 3D point pt into unit scale (ie, range [0,1]) assuming that pt falls in the
# the range [RANGE_MIN, RANGE_MAX]. Value is clipped [0,1]
RANGE_MIN = np.array((-250.0, 50, -200))
RANGE_MAX = np.array((250.0, 600, 250))
def scale_point(pt):
    pt = (pt - RANGE_MIN) / (RANGE_MAX - RANGE_MIN)
    pt = np.clip(pt, 0, 1)
    return pt


# show a single hand graphically
class MainWidget2(BaseWidget) :
    def __init__(self):
        super(MainWidget2, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()

        self.hand_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.hand_disp)

    def on_update(self) :
        leap_frame = self.leap.frame()

        pt = leap_one_palm(leap_frame)

        norm_pt = scale_point(pt)

        self.hand_disp.set_pos(norm_pt)

        self.label.text = leap_info(self.leap)
        self.label.text += 'x:%d\ny:%d\nz:%d\n' % (pt[0], pt[1], pt[2])
        self.label.text += 'x:%.2f\ny:%.2f\nz:%.2f\n' % (norm_pt[0], norm_pt[1], norm_pt[2])


# Show two hands
class MainWidget3(BaseWidget) :
    def __init__(self):
        super(MainWidget3, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()

        self.left_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.left_disp)
        self.right_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.6, .2, .2))
        self.canvas.add(self.right_disp)

    def on_update(self) :
        leap_frame = self.leap.frame()

        pt1, pt2 = leap_two_palms(leap_frame)

        norm_pt1 = scale_point(pt1)
        norm_pt2 = scale_point(pt2)

        self.left_disp.set_pos(norm_pt1)
        self.right_disp.set_pos(norm_pt2)

        self.label.text = leap_info(self.leap)
        self.label.text += 'x:%.2f\ny:%.2f\nz:%.2f\n' % (norm_pt1[0], norm_pt1[1], norm_pt1[2])
        self.label.text += 'x:%.2f\ny:%.2f\nz:%.2f\n' % (norm_pt2[0], norm_pt2[1], norm_pt2[2])



# show 5 finger positions of a single hand graphically
class MainWidget4(BaseWidget) :
    def __init__(self):
        super(MainWidget4, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()

        self.finger_disp = []
        for x in range(5):
            disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2), size_range=(5, 20))
            self.canvas.add(disp)
            self.finger_disp.append(disp)

    def on_update(self) :
        leap_frame = self.leap.frame()

        pts = leap_fingers_fingers(leap_frame)
        norm_pts = [scale_point(pt) for pt in pts]

        for i, pt in enumerate(norm_pts):
            self.finger_disp[i].set_pos(pt)

        self.label.text = leap_info(self.leap)



# Demonstrate the Clap Gesture: Tracks the 2 hands and displays the Pop objects
# when the hands clap
class MainWidget5(BaseWidget) :
    def __init__(self):
        super(MainWidget5, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()

        self.left_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.left_disp)
        self.right_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.6, .2, .2))
        self.canvas.add(self.right_disp)

        self.anims = AnimGroup()
        self.canvas.add(self.anims)

        self.clap = ClapGesture(self.on_clap)

    def on_update(self) :
        leap_frame = self.leap.frame()

        pt1, pt2 = leap_two_palms(leap_frame)

        # feed data to clap gesture detector
        self.clap.on_update(pt1, pt2)

        norm_pt1 = scale_point(pt1)
        norm_pt2 = scale_point(pt2)

        self.left_disp.set_pos(norm_pt1)
        self.right_disp.set_pos(norm_pt2)

        self.label.text = leap_info(self.leap)
        self.label.text += 'x:%d\ny:%d\nz:%d\n' % (pt1[0], pt1[1], pt1[2])
        self.label.text += 'x:%d\ny:%d\nz:%d\n' % (pt2[0], pt2[1], pt2[2])
        self.label.text += 'dist = %d\n' % np.linalg.norm(pt1 - pt2)

        self.anims.on_update()

    # callback happens when a clap is detected.
    def on_clap(self, pos):
        norm_pos = scale_point(pos)[0:2]

        # convert to screen coordinates for display
        screen_pos = norm_pos * np.array([Window.width/2,Window.height/2]) + np.array([10,10])

        # create a graphic that shows the clap
        pop = Pop(screen_pos)
        self.anims.add(pop)


# A clap gesture detector. Notifies a callback when a clap is detected.
# the callback function takes a single argument: pos (3D point)
class ClapGesture(object):
    def __init__(self, callback):
        super(ClapGesture, self).__init__()
        self.callback = callback

        self.clap_thresh = 70
        self.unclap_thresh = 100

        self.is_clapped = False

    def on_update(self, left_pt, right_pt):

        dist = np.linalg.norm( left_pt - right_pt )
        if dist < self.clap_thresh and not self.is_clapped:
            print "clap!!"

            mid_point = (left_pt + right_pt) / 2

            self.callback(mid_point)

            self.is_clapped = True

        if dist > self.unclap_thresh and self.is_clapped:
            self.is_clapped = False
            print "UN clap!!"


# An animation / display to go along with the Clap Gesture
class Pop(InstructionGroup) :
    def __init__(self, pos):
        super(Pop, self).__init__()

        self.color = Color(1,0,0,1)
        self.add(self.color)

        self.circle = CEllipse(cpos = pos)
        self.add(self.circle)

        # create keyframe animation: a frame is: (time, alpha radius)
        self.anim = KFAnim((0,   1, 30),
                           (0.5, 0, 60),)

        self.time = 0
        self.on_update(0)

    def on_update(self, dt):
        values = self.anim.eval(self.time)
        self.color.a = values[0]
        self.circle.csize = values[1] * 2, values[1] * 2
        self.time += dt

        return self.anim.is_active(self.time)



# Example using Leap's built-in Circle Gesture
class MainWidget6(BaseWidget) :
    def __init__(self):
        super(MainWidget6, self).__init__()

        self.label = topleft_label()
        self.add_widget(self.label)

        self.leap = Leap.Controller()
        self.leap.enable_gesture(Leap.Gesture.TYPE_CIRCLE)

        self.hand_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.hand_disp)

        self.expired_gestures = []

        self.anims = AnimGroup()
        self.canvas.add(self.anims)

    def _on_circle(self, pt) :
        print 'CIRCLE'
        norm_pt = scale_point(pt)[0:2]

        # convert to screen coordinates for display
        screen_pos = norm_pt * np.array([Window.width/2,Window.height/2]) + np.array([10,10])

        # create a graphic that shows the clap
        pop = Pop(screen_pos)
        self.anims.add(pop)

    def on_update(self) :
        self.anims.on_update()
        leap_frame = self.leap.frame()

        pt = leap_one_palm(leap_frame)
        norm_pt = scale_point(pt)

        self.hand_disp.set_pos(norm_pt)

        # leap_gesture(leap_frame)
        circles = [Leap.CircleGesture(g) for g in leap_frame.gestures() if g.type is Leap.Gesture.TYPE_CIRCLE]

        if len(circles) == 0:
            self.expired_gestures = []

        self.label.text = leap_info(self.leap)
        for g in circles:
            self.label.text += 'circle id:%d state:%d prog:%.1f\n' % (g.id, g.state, g.progress)

            if g.progress > 1.2 and g.id not in self.expired_gestures:
                self.expired_gestures.append(g.id)
                self._on_circle(pt)


# pass in which MainWidget to run as a command-line arg
run(eval('MainWidget' + sys.argv[1]))
